@@ -1,7 +1,11 @@
 # noinspection PyUnresolvedReferences
 from .. import *
+from .. import dispatch as dispatch_original
+from ..shape import dispatch_unwrap_dimensions
 from ..types import Number, NPNumeric, TFNumeric
 
+dispatch = dispatch_unwrap_dimensions(dispatch_original)
+
 Numeric = {Number, NPNumeric, TFNumeric}
 
 from .generic import *
@@ -10,7 +14,6 @@ from .linear_algebra import *
 from .random import *
 
 # noinspection PyUnresolvedReferences
-import tensorflow as tf
 from ..types import _tf_retrievables
 
 # Retrieve types.
