@@ -18,25 +18,27 @@ from lab.custom import (
 from lab.tensorflow.custom import as_tf
 from lab.torch.custom import as_torch
 from lab.jax.custom import as_jax
-from .util import approx
+
+# noinspection PyUnresolvedReferences
+from .util import allclose, check_lazy_shapes
 
 
-def test_as_tf():
+def test_as_tf(check_lazy_shapes):
     assert isinstance(as_tf(B.randn()), B.TFNumeric)
     assert isinstance(as_tf((B.randn(),))[0], B.TFNumeric)
 
 
-def test_as_torch():
+def test_as_torch(check_lazy_shapes):
     assert isinstance(as_torch(B.randn()), B.TorchNumeric)
     assert isinstance(as_torch((B.randn(),))[0], B.TorchNumeric)
 
 
-def test_as_jax():
+def test_as_jax(check_lazy_shapes):
     assert isinstance(as_jax(B.randn()), B.JaxNumeric)
     assert isinstance(as_jax((B.randn(),))[0], B.JaxNumeric)
 
 
-def check_grad(f, args, kw_args=None, digits=6):
+def check_grad(f, args, kw_args=None, rtol=1e-8):
     """Check the gradients of a function.
 
     Args:
@@ -44,8 +46,7 @@ def check_grad(f, args, kw_args=None, digits=6):
         args (tuple): Arguments to check `f` at.
         kw_args (tuple, optional): Keyword arguments to check `f` at. Defaults
             to no keyword arguments.
-        digits (int, optional): Number of digits accuracy to check. Defaults to
-            `6`.
+        rtol (float, optional): Relative tolerance. Defaults to `1e-8`.
     """
     # Default to no keyword arguments.
     if kw_args is None:
@@ -69,7 +70,7 @@ def check_grad(f, args, kw_args=None, digits=6):
 
         # Check AutoGrad gradient.
         autograd_grad = grad(f_i)(args[i])
-        approx(numerical_grad, autograd_grad, digits)
+        allclose(numerical_grad, autograd_grad, rtol=rtol)
 
         # Check TensorFlow gradient.
         tf_args = tuple([as_tf(arg) for arg in args])
@@ -77,22 +78,22 @@ def check_grad(f, args, kw_args=None, digits=6):
         with tf.GradientTape() as t:
             t.watch(tf_args[i])
             tf_grad = t.gradient(f_i(tf_args[i]), tf_args[i]).numpy()
-        approx(numerical_grad, tf_grad, digits)
+        allclose(numerical_grad, tf_grad, rtol=rtol)
 
         # Check PyTorch gradient.
         torch_args = tuple([as_torch(arg, grad=True) for arg in args])
         f_i = create_f_i(i, torch_args)
         f_i(torch_args[i]).backward()
-        approx(numerical_grad, torch_args[i].grad, digits)
+        allclose(numerical_grad, torch_args[i].grad, rtol=rtol)
 
         # Check Jax gradient.
         torch_args = tuple([jax.device_put(arg) for arg in args])
         f_i = create_f_i(i, torch_args)
         jax_grad = jax.grad(f_i)(args[i])
-        approx(numerical_grad, jax_grad, digits)
+        allclose(numerical_grad, jax_grad, rtol=rtol)
 
 
-def test_toeplitz_solve():
+def test_toeplitz_solve(check_lazy_shapes):
     check_sensitivity(
         toeplitz_solve, s_toeplitz_solve, (B.randn(3), B.randn(2), B.randn(3))
     )
@@ -103,18 +104,18 @@ def test_toeplitz_solve():
     check_grad(toeplitz_solve, (B.randn(3), B.randn(2), B.randn(3, 4)))
 
 
-def test_bvn_cdf():
+def test_bvn_cdf(check_lazy_shapes):
     check_sensitivity(bvn_cdf, s_bvn_cdf, (B.rand(3), B.rand(3), B.rand(3)))
     check_grad(bvn_cdf, (B.rand(3), B.rand(3), B.rand(3)))
 
 
-def test_expm():
+def test_expm(check_lazy_shapes):
     check_sensitivity(expm, s_expm, (B.randn(3, 3),))
     check_grad(expm, (B.randn(3, 3),))
 
 
 @pytest.mark.xfail
-def test_logm():
+def test_logm(check_lazy_shapes):
     mat = B.eye(3) + 0.1 * B.randn(3, 3)
     check_sensitivity(logm, s_logm, (mat,))
     check_grad(logm, (mat,))
